/*
 * Bluetooth Controlled 8x8 RGB LED Matrix
 *
 * Example Pattern class definitions
 *
 * Written by: Craig A. Lindley
 * Last Update: 11/11/2014
 */

#include "BaseClass.h"
#include "Font.h"
#include "Types.h"

// Fixup forward references
extern void clearFrameBuffer();
extern void fillFrameBuffer(rgb24 color);
extern rgb24 createHSVColor(float hue, float saturation,  float value);
extern rgb24 createHSVColorWithDivisions(int divisions, int index);
extern void setPixel(byte col, byte row, rgb24 color);
extern void setPixel(byte pixelNumber, rgb24 color);
extern void setPixel(byte col, byte row, int redPWM, int greenPWM, int bluePWM);
extern rgb24 colorWheel(int wheelPos);

#define MATRIX_WIDTH  8
#define MATRIX_HEIGHT 8

#ifndef PATTERNS_H
#define PATTERNS_H

// Ring data from inner to outer in POINTS 
POINT ring0 [] = {
    3, 4, 4, 4, 3, 3, 4, 3};
POINT ring1 [] = {
    2, 5, 3, 5, 4, 5, 5, 5,
    2, 4, 5, 4, 2, 3, 5, 3,
    2, 2, 3, 2, 4, 2, 5, 2};
POINT ring2 [] = {
    1, 6, 2, 6, 3, 6, 4, 6,
    5, 6, 6, 6, 1, 5, 6, 5,
    1, 4, 6, 4, 1, 3, 6, 3,
    1, 2, 6, 2, 1, 1, 2, 1,
    3, 1, 4, 1, 5, 1, 6, 1};
POINT ring3 [] = {
    0, 7, 1, 7, 2, 7, 3, 7,
    4, 7, 5, 7, 6, 7, 7, 7,
    0, 6, 7, 6, 0, 5, 7, 5,
    0, 4, 7, 4, 0, 3, 7, 3,
    0, 2, 7, 2, 0, 1, 7, 1,
    0, 0, 1, 0, 2, 0, 3, 0,
    4, 0, 5, 0, 6, 0, 7, 0};

// Spiral from inside to outside in led number
byte spiralMap [] = {
    35, 36, 28, 27, 26, 34, 42, 43, 44, 45, 37, 29, 21, 20, 19, 18,
    17, 25, 33, 41, 49, 50, 51, 52, 53, 54, 46, 38, 30, 22, 14, 13,
    12, 11, 10,  9,  8, 16, 24, 32, 40, 48, 56, 57, 58, 59, 60, 61,
    62, 63, 55, 47, 39, 31, 23, 15,  7,  6,  5,  4,  3,  2,  1,  0   
};

byte RING_POINTS [28] = {
    0,  1,  2,  3,  4,  5,  6, 7,
    15, 23, 31, 39, 47, 55, 63,
    62, 61, 60, 59, 58, 57, 56,
    48, 40, 32, 24, 16,  8,
};

// Set a specified ring to the specified color
void setRingToColor(int ringNumber, rgb24 color) {

    if (ringNumber < 0) {
        ringNumber = 0;
    }    
    else if (ringNumber > 3) {
        ringNumber = 3;
    }

    switch (ringNumber) {
    case 0:
        for (int i = 0; i < 4; i++) {
            POINT pt = ring0[i];
            setPixel(pt.col, pt.row, color);
        }
        break;

    case 1:
        for (int i = 0; i < 12; i++) {
            POINT pt = ring1[i];
            setPixel(pt.col, pt.row, color);
        }
        break;

    case 2:
        for (int i = 0; i < 20; i++) {
            POINT pt = ring2[i];
            setPixel(pt.col, pt.row, color);
        }
        break;

    case 3:
        for (int i = 0; i < 28; i++) {
            POINT pt = ring3[i];
            setPixel(pt.col, pt.row, color);
        }
        break;
    }
}

// Draw a horizontal line across display
void drawHorizLine(byte y, rgb24 color) {
    for (int x = 0; x < MATRIX_WIDTH; x++) {
        setPixel(x, y, color);
    }
}

// Draw a vertical line across display
void drawVertLine(byte x, rgb24 color) {
    for (int y = 0; y < MATRIX_HEIGHT; y++) {
        setPixel(x, y, color);
    }
}

void drawLine(int x0, int y0, int x1, int y1, rgb24 color) {

    int dx = abs(x1 - x0), sx = x0 < x1 ? 1 : -1;
    int dy = -abs(y1 - y0), sy = y0 < y1 ? 1 : -1;
    int err = dx + dy, e2;
    for (;;){
        setPixel(x0, y0, color);
        if (x0 == x1 && y0 == y1) break;
        e2 = 2 * err;
        if (e2 > dy) { 
            err += dy; 
            x0 += sx; 
        }
        if (e2 < dx) { 
            err += dx; 
            y0 += sy; 
        }
    }
}

/*******************************************************************/
/***                        Pattern Classes                      ***/
/*******************************************************************/

class Rotation1 : 
public BaseClass {

    enum RState {
        STATE_DRAW, STATE_DELAY
    };
    enum RState state;

    long delayTime;
    int delayTimeMS;

    int forwardIndex, reverseIndex;
    int colorSelector;

    byte pt;
    rgb24 color;

public:

    Rotation1(GLOBAL_PARAMETERS *_ptrParms) : 
    BaseClass(_ptrParms) {
    }

    void initStateMachine() {

        // Calculate timeout delay
        delayTimeMS = map(ptrParms->speed, 1, 10, 500, 1);

        forwardIndex =  0;
        reverseIndex = 14;

        colorSelector = 0;

        state = STATE_DRAW;
    }

    void runStateMachine(boolean updated) {

        if (updated) {
            // Calculate timeout delay
            delayTimeMS = map(ptrParms->speed, 1, 10, 500, 1);
        }

        switch (state) {

        case STATE_DRAW:

            switch (colorSelector) {
            case 0:
                color = COLOR_RED;
                break;

            case 1:
                color = COLOR_GREEN;
                break;

            case 2:
                color = COLOR_BLUE;
                break;

            case 3:
                color = COLOR_WHITE;
                break;
            }
            // Erase points
            clearFrameBuffer();

            // Set middle color
            setRingToColor(1, COLOR_BLUE);

            // Draw forward point
            pt = RING_POINTS[forwardIndex++];
            setPixel(pt, color);
            if (forwardIndex >= 28) {
                forwardIndex = 0;

                // Update color selector
                colorSelector++;
                if (colorSelector >= 4) {
                    colorSelector = 0;
                }
            }

            // Draw reverse point
            pt = RING_POINTS[reverseIndex--];
            setPixel(pt, color);
            if (reverseIndex < 0) {
                reverseIndex = 27;
            }

            // Calculate delay time
            delayTime = millis() + delayTimeMS;

            state = STATE_DELAY;
            break;

        case STATE_DELAY:
            if (millis() >= delayTime) {
                state = STATE_DRAW;
            }
            break;
        }
    }
};

class ColoredRings : 
public BaseClass {

    enum RRState {
        STATE_DRAW_RING, STATE_CLEAR_RING, STATE_DELAY
    };
    enum RRState state;

    long delayTime;
    int delayTimeMS;
    int colorIndex;
    int ringIndex;
    boolean drawingRing;

public:

    ColoredRings(GLOBAL_PARAMETERS *_ptrParms) : 
    BaseClass(_ptrParms) {
    }

    void initStateMachine() {
        // Calculate timeout delay
        delayTimeMS = map(ptrParms->speed, 1, 10, 1100, 5);

        colorIndex = 0;
        ringIndex = 0;
        state = STATE_DRAW_RING;
    }

    void runStateMachine(boolean updated) {

        rgb24 color;

        if (updated) {
            // Calculate new timeout delay
            delayTimeMS = map(ptrParms->speed, 1, 10, 1100, 5);
        }

        switch (state) {

        case STATE_DRAW_RING:
            drawingRing = true;
            color = colorWheel(colorIndex);
            colorIndex += 16;
            setRingToColor(ringIndex++, color);

            if (ringIndex > 3) {
                ringIndex = 0;
                state = STATE_CLEAR_RING;
            }    
            else     {
                delayTime = millis() + delayTimeMS;
                state = STATE_DELAY;
            }
            break;

        case STATE_CLEAR_RING:
            drawingRing = false;
            setRingToColor(ringIndex++, COLOR_BLACK);

            if (ringIndex > 3) {
                ringIndex = 0;
                state = STATE_DRAW_RING;
            }    
            else     {
                delayTime = millis() + delayTimeMS;
                state = STATE_DELAY;
            }
            break;


        case STATE_DELAY:
            if (millis() > delayTime) {
                if (drawingRing) {
                    state = STATE_DRAW_RING;
                }    
                else    {
                    state = STATE_CLEAR_RING;
                }
            }
            break;
        }
    }
};

class Rainbow1 : 
public BaseClass {

    enum RRState {
        STATE_INIT, STATE_TEST, STATE_DRAW, STATE_DELAY
    };
    enum RRState state;

    long delayTime;
    int delayTimeMS;
    int outerIndex;

public:

    Rainbow1(GLOBAL_PARAMETERS *_ptrParms) : 
    BaseClass(_ptrParms) {
    }

    void initStateMachine() {
        // Calculate timeout delay
        delayTimeMS = map(ptrParms->speed, 1, 10, 600, 2);

        state = STATE_INIT;
    }

    void runStateMachine(boolean updated) {

        rgb24 color;

        if (updated) {
            // Calculate new timeout delay
            delayTimeMS = map(ptrParms->speed, 1, 10, 600, 2);
        }

        switch (state) {

        case STATE_INIT:
            outerIndex = 0;
            state = STATE_TEST;
            break;

        case STATE_TEST:
            if (outerIndex >= 256) {
                state = STATE_INIT;
            }    
            else    {
                state = STATE_DRAW;
            }
            break;

        case STATE_DRAW:
            for (int i = 0; i < 64; i++) {
                color = colorWheel((i + outerIndex) % 256);
                setPixel(spiralMap[i], color);
            }
            outerIndex++;
            delayTime = millis() + delayTimeMS;
            state = STATE_DELAY;
            break;

        case STATE_DELAY:
            if (millis() >= delayTime) {
                state = STATE_TEST;
            }            
            break;
        }
    }
};

class Rainbow2 : 
public BaseClass {

    enum RRState {
        STATE_INIT, STATE_TEST, STATE_DRAW, STATE_DELAY
    };
    enum RRState state;

    long delayTime;
    int delayTimeMS;
    int outerIndex;

public:

    Rainbow2(GLOBAL_PARAMETERS *_ptrParms) : 
    BaseClass(_ptrParms) {
    }

    void initStateMachine() {
        // Calculate timeout delay
        delayTimeMS = map(ptrParms->speed, 1, 10, 200, 1);

        state = STATE_INIT;
    }

    void runStateMachine(boolean updated) {

        rgb24 color;

        if (updated) {
            // Calculate new timeout delay
            delayTimeMS = map(ptrParms->speed, 1, 10, 200, 1);
        }

        switch (state) {

        case STATE_INIT:
            outerIndex = 0;
            state = STATE_TEST;
            break;

        case STATE_TEST:
            if (outerIndex >= 256 * 5) {
                state = STATE_INIT;
            }    
            else    {
                state = STATE_DRAW;
            }
            break;

        case STATE_DRAW:
            for (int i = 0; i < 64; i++) {
                color = colorWheel(((i * 256 / 64) + outerIndex) % 256);
                setPixel(spiralMap[i], color);
            }
            outerIndex++;
            delayTime = millis() + delayTimeMS;
            state = STATE_DELAY;
            break;

        case STATE_DELAY:
            if (millis() >= delayTime) {
                state = STATE_TEST;
            }            
            break;
        }
    }
};

class Plasma : 
public BaseClass {

    enum PState {
        STATE_DRAW, STATE_DELAY
    };
    enum PState state;

    long delayTime;
    int delayTimeMS;

    float time;
    int cycles;

public:

    Plasma(GLOBAL_PARAMETERS *_ptrParms) : 
    BaseClass(_ptrParms) {
    }

    void initStateMachine() {

        time = 6;
        cycles = 0;

        // Calculate timeout delay
        delayTimeMS = map(ptrParms->speed, 1, 10, 1500, 100);

        state = STATE_DRAW;
    }

    void runStateMachine(boolean updated) {

        if (updated) {
            // Calculate timeout delay
            delayTimeMS = map(ptrParms->speed, 1, 10, 1500, 100);
        }

        switch (state) {
        case STATE_DRAW:

            for (int y = 0; y < MATRIX_HEIGHT; y++) {
                for (int x = 0; x < MATRIX_WIDTH; x++) {
                    float v = 0;
                    float wibble = sin(time);
                    v += sin(x * wibble * 2 + time);
                    v += cos(y * (128 - wibble) * 2 + time);
                    v += sin(y * x * cos(-time) / 2);

                    v += 3.0;
                    v /= 6.0;
                    v *= 256;

                    // Create color
                    rgb24 color = colorWheel(v);

                    // Set the pixel
                    setPixel(x, y, color);
                }
            }
            time += 0.1;
            cycles++;

            if (cycles >= 2048) {
                time = 0;
                cycles = 0;
            }

            // Calculate delay time
            delayTime = millis() + delayTimeMS;

            state = STATE_DELAY;
            break;

        case STATE_DELAY:
            if (millis() >= delayTime) {
                state = STATE_DRAW;
            }
            break;
        }
    }
};

class Fade : 
public BaseClass {

    enum FState {
        STATE_DRAW, STATE_DELAY
    };
    enum FState state;

    long delayTime;
    int delayTimeMS;

    int colorIndex;
    rgb24 color;

public:

    Fade(GLOBAL_PARAMETERS *_ptrParms) : 
    BaseClass(_ptrParms) {
    }

    void initStateMachine() {

        // Calculate timeout delay
        delayTimeMS = map(ptrParms->speed, 1, 10, 250, 1);

        colorIndex = random(0, 256);

        state = STATE_DRAW;
    }

    void runStateMachine(boolean updated) {

        if (updated) {
            // Calculate timeout delay
            delayTimeMS = map(ptrParms->speed, 1, 10, 250, 1);
        }

        switch (state) {
        case STATE_DRAW:
            // Create color
            color = colorWheel(colorIndex);

            // Fill the matrix display
            fillFrameBuffer(color);

            colorIndex++;

            // Calculate delay time
            delayTime = millis() + delayTimeMS;

            state = STATE_DELAY;
            break;

        case STATE_DELAY:
            if (millis() >= delayTime) {
                state = STATE_DRAW;
            }
            break;
        }
    }
};

class Spiral1 : 
public BaseClass {

    enum SState {
        STATE_DRAW, STATE_DELAY
    };
    enum SState state;

    long delayTime;
    int delayTimeMS;

    int colorIndex;
    rgb24 color;

    byte lastLED;
    byte nextLED;

public:

    Spiral1(GLOBAL_PARAMETERS *_ptrParms) : 
    BaseClass(_ptrParms) {
    }

    void initStateMachine() {

        // Calculate timeout delay
        delayTimeMS = map(ptrParms->speed, 1, 10, 300, 30);

        colorIndex = random(0, 256);

        nextLED = 0;
        lastLED = 60;

        state = STATE_DRAW;
    }

    void runStateMachine(boolean updated) {

        if (updated) {
            // Calculate timeout delay
            delayTimeMS = map(ptrParms->speed, 1, 10, 300, 30);
        }

        switch (state) {
        case STATE_DRAW:

            // Erase last LED
            setPixel(spiralMap[lastLED], COLOR_BLACK);
            lastLED++;
            if (lastLED >= 64) {
                lastLED = 0;
            }

            // Create color
            color = colorWheel(colorIndex);

            // Draw next LED
            setPixel(spiralMap[nextLED], color);
            nextLED++;
            if (nextLED >= 64) {
                nextLED = 0;
            }

            colorIndex += 8;

            // Calculate delay time
            delayTime = millis() + delayTimeMS;

            state = STATE_DELAY;
            break;

        case STATE_DELAY:
            if (millis() >= delayTime) {
                state = STATE_DRAW;
            }
            break;
        }
    }
};

class Chaser : 
public BaseClass {

    enum SState {
        STATE_DRAW, STATE_DELAY
    };
    enum SState state;

    long delayTime;
    int delayTimeMS;

    int colorIndex;
    rgb24 color;

    POINT tail[4], p;
    int x, y;

    void shiftPoints() {
        tail[3] = tail[2];
        tail[2] = tail[1];
        tail[1] = tail[0];
    }

public:

    Chaser(GLOBAL_PARAMETERS *_ptrParms) : 
    BaseClass(_ptrParms) {
    }

    void initStateMachine() {

        // Calculate timeout delay
        delayTimeMS = map(ptrParms->speed, 1, 10, 300, 30);

        colorIndex = random(0, 256);

        memset(tail, 0, sizeof(tail));

        // Pick a random starting point
        x = random(0, MATRIX_WIDTH);
        y = random(0, MATRIX_HEIGHT);

        state = STATE_DRAW;
    }

    void runStateMachine(boolean updated) {

        if (updated) {
            // Calculate timeout delay
            delayTimeMS = map(ptrParms->speed, 1, 10, 300, 30);
        }

        switch (state) {
        case STATE_DRAW:
            // Erase last LED
            setPixel(tail[3].col, tail[3].row, COLOR_BLACK);
            shiftPoints();

            // Find new point
            if (random(2) == 0) {
                // Go vertical
                if (random(2) == 0) {
                    // Go up
                    y++;
                    if (y >= MATRIX_HEIGHT) {
                        y = 0;
                    }
                }    
                else    {
                    // Go down
                    y--;
                    if (y < 0) {
                        y = MATRIX_HEIGHT - 1;
                    }
                }
            }    
            else    {
                // Go horizontal
                if (random(2) == 0) {
                    // Go left
                    x--;
                    if (x < 0) {
                        x = MATRIX_WIDTH - 1;
                    }
                }    
                else    {
                    // Go right
                    x++;
                    if (x >= MATRIX_WIDTH) {
                        x = 0;
                    }
                }
            }
            // Create color
            color = colorWheel(colorIndex);
            colorIndex += 8;

            // Draw next LED
            setPixel(x, y, color);

            p.col = x;
            p.row = y;

            // Place new point in array
            tail[0] = p;

            // Calculate delay time
            delayTime = millis() + delayTimeMS;

            state = STATE_DELAY;
            break;

        case STATE_DELAY:
            if (millis() >= delayTime) {
                state = STATE_DRAW;
            }
            break;
        }
    }
};

class Lines : 
public BaseClass {

    enum LState {
        STATE_INIT, STATE_TEST_COUNT, STATE_DRAW, STATE_DELAY
    };
    enum LState state;

    long delayTime;
    int delayTimeMS;

    rgb24 color;
    int x0, x1, y0, y1;
    int count;

public:

    Lines(GLOBAL_PARAMETERS *_ptrParms) : 
    BaseClass(_ptrParms) {
    }

    void initStateMachine() {

        // Calculate timeout delay
        delayTimeMS = map(ptrParms->speed, 1, 10, 300, 30);

        state = STATE_INIT;
    }

    void runStateMachine(boolean updated) {

        if (updated) {
            // Calculate timeout delay
            delayTimeMS = map(ptrParms->speed, 1, 10, 300, 30);
        }

        switch (state) {

        case STATE_INIT:
            count = 15;
            state = STATE_TEST_COUNT;
            break;

        case STATE_TEST_COUNT:
            if (count-- < 0) {
                // Erase the display
                clearFrameBuffer();

                state = STATE_INIT;
            }    
            else    {
                state = STATE_DRAW;
            }
            break;

        case STATE_DRAW:

            // Create color
            color = colorWheel(random(256));

            if (random(2) == 1) {
                // X is fixed
                x0 = 0;
                x1 = 7;
                y0 = random(0, MATRIX_HEIGHT);
                y1 = random(0, MATRIX_HEIGHT);
            }    
            else    {
                // Y is fixed
                y0 = 0;
                y1 = 7;

                x0 = random(0, MATRIX_WIDTH);
                x1 = random(0, MATRIX_WIDTH);

            }
            // Draw a line
            drawLine(x0, y0, x1, y1, color);

            // Calculate delay time
            delayTime = millis() + delayTimeMS;

            state = STATE_DELAY;
            break;

        case STATE_DELAY:
            if (millis() >= delayTime) {
                state = STATE_TEST_COUNT;
            }
            break;
        }
    }
};

class RingChase : 
public BaseClass {

    enum RCState {
        STATE_DRAW, STATE_DELAY
    };
    enum RCState state;

    long delayTime;
    int delayTimeMS;

    rgb24 color;
    int prevPoint, nextPoint;
    int pointIndex;
    int colorIndex;

public:

    RingChase(GLOBAL_PARAMETERS *_ptrParms) : 
    BaseClass(_ptrParms) {
    }

    void initStateMachine() {

        // Calculate timeout delay
        delayTimeMS = map(ptrParms->speed, 1, 10, 500, 1);

        prevPoint = 0;
        prevPoint = 0;
        pointIndex = 0;
        colorIndex = random(0, 96);

        state = STATE_DRAW;
    }

    void runStateMachine(boolean updated) {

        if (updated) {
            // Calculate timeout delay
            delayTimeMS = map(ptrParms->speed, 1, 10, 500, 1);
        }

        switch (state) {

        case STATE_DRAW:
            // Erase previous point
            setPixel(prevPoint, COLOR_BLACK);

            // Get next point to draw
            nextPoint = RING_POINTS[pointIndex++];
            if (pointIndex >= 28) {
                pointIndex = 0;
            }

            // Create a color
            color = createHSVColorWithDivisions(96, colorIndex++) ;

            // Draw new point
            setPixel(nextPoint, color);

            prevPoint = nextPoint;

            // Calculate delay time
            delayTime = millis() + delayTimeMS;

            state = STATE_DELAY;
            break;

        case STATE_DELAY:
            if (millis() >= delayTime) {
                state = STATE_DRAW;
            }
            break;
        }
    }
};

class RingChase2 : 
public BaseClass {

    float HUE_ARRAY [8] = {
        0,  32,  64,  96,
        128, 160, 192, 224
    };

    byte RING_POINTS [29] = {
        0,  1,  2,  3,  4,  5,  6, 7,
        15, 23, 31, 39, 47, 55, 63,
        62, 61, 60, 59, 58, 57, 56,
        48, 40, 32, 24, 16,  8,  0
    };

    enum RCState {
        STATE_DRAW, STATE_DELAY
    };
    enum RCState state;

    long delayTime;
    int delayTimeMS;

    rgb24 color;
    int prevPoint, nextPoint;
    int pointIndex;
    int hueIndex;
    float hue;

public:

    RingChase2(GLOBAL_PARAMETERS *_ptrParms) : 
    BaseClass(_ptrParms) {
    }

    void initStateMachine() {

        // Calculate timeout delay
        delayTimeMS = map(ptrParms->speed, 1, 10, 500, 1);

        prevPoint = 0;
        prevPoint = 0;
        pointIndex = 0;
        hueIndex = 0;

        state = STATE_DRAW;
    }

    void runStateMachine(boolean updated) {

        if (updated) {
            // Calculate timeout delay
            delayTimeMS = map(ptrParms->speed, 1, 10, 500, 1);
        }

        switch (state) {

        case STATE_DRAW:
            // Erase previous point
            setPixel(prevPoint, COLOR_BLACK);

            // Get next point to draw
            nextPoint = RING_POINTS[pointIndex++];
            if (pointIndex >= 29) {
                pointIndex = 0;
                hueIndex++;
                if (hueIndex >= 8) {
                    hueIndex= 0;
                }
            }

            // Create a color
            hue = HUE_ARRAY[hueIndex];
            color = createHSVColor(hue, 1.0, 1.0);

            // Draw new point
            setPixel(nextPoint, color);

            prevPoint = nextPoint;

            // Calculate delay time
            delayTime = millis() + delayTimeMS;

            state = STATE_DELAY;
            break;

        case STATE_DELAY:
            if (millis() >= delayTime) {
                state = STATE_DRAW;
            }
            break;
        }
    }
};

class RandomPTS : 
public BaseClass {

    enum RPState {
        STATE_DRAW, STATE_DELAY
    };
    enum RPState state;

    long delayTime;
    int delayTimeMS;

    int col, row, redPWM, greenPWM, bluePWM;

public:

    RandomPTS(GLOBAL_PARAMETERS *_ptrParms) : 
    BaseClass(_ptrParms) {
    }

    void initStateMachine() {

        // Calculate timeout delay
        delayTimeMS = map(ptrParms->speed, 1, 10, 500, 1);

        state = STATE_DRAW;
    }

    void runStateMachine(boolean updated) {

        if (updated) {
            // Calculate timeout delay
            delayTimeMS = map(ptrParms->speed, 1, 10, 500, 1);
        }

        switch (state) {

        case STATE_DRAW:
            // Pick a pixel
            col = random(0, MATRIX_WIDTH);
            row = random(0, MATRIX_HEIGHT);

            // Create a color
            redPWM   = random(0, 4096);
            greenPWM = random(0, 4096);
            bluePWM  = random(0, 4096);

            // Draw new point
            setPixel(col, row, redPWM, greenPWM, bluePWM);

            // Calculate delay time
            delayTime = millis() + delayTimeMS;

            state = STATE_DELAY;
            break;

        case STATE_DELAY:
            if (millis() >= delayTime) {
                state = STATE_DRAW;
            }
            break;
        }
    }
};

#endif








